@@ -5,10 +5,11 @@ import tempfile
 import warnings
 
 from octue import exceptions
+from octue.cloud import storage
+from octue.cloud.storage import GoogleCloudStorageClient
 from octue.resources import Datafile, Dataset
 from octue.resources.filter_containers import FilterSet
-from octue.utils.cloud import storage
-from octue.utils.cloud.storage.client import GoogleCloudStorageClient
+from tests import TEST_BUCKET_NAME
 from tests.base import BaseTestCase
 
 
@@ -270,10 +271,10 @@ class DatasetTestCase(BaseTestCase):
             self.assertEqual(len(filtered_files), 0)
 
     def test_hash_value(self):
-        """ Test hashing a dataset with multiple files gives a hash of length 128. """
+        """Test hashing a dataset with multiple files gives a hash of length 8."""
         hash_ = self.create_valid_dataset().hash_value
         self.assertTrue(isinstance(hash_, str))
-        self.assertTrue(len(hash_) == 64)
+        self.assertTrue(len(hash_) == 8)
 
     def test_hashes_for_the_same_dataset_are_the_same(self):
         """ Ensure the hashes for two datasets that are exactly the same are the same."""
@@ -286,14 +287,25 @@ class DatasetTestCase(BaseTestCase):
         dataset = self.create_valid_dataset()
         self.assertEqual(len(dataset.serialise()["files"]), 2)
 
+    def test_is_in_cloud(self):
+        """Test whether all files of a dataset are in the cloud or not can be determined."""
+        self.assertFalse(Dataset().all_files_are_in_cloud)
+        self.assertFalse(self.create_valid_dataset().all_files_are_in_cloud)
+
+        files = [
+            Datafile(timestamp=None, path="gs://hello/file.txt"),
+            Datafile(timestamp=None, path="gs://goodbye/file.csv"),
+        ]
+
+        self.assertTrue(Dataset(files=files).all_files_are_in_cloud)
+
     def test_from_cloud(self):
         """Test that a Dataset in cloud storage can be accessed."""
         project_name = "test-project"
-        bucket_name = os.environ["TEST_BUCKET_NAME"]
 
-        with tempfile.TemporaryDirectory() as output_directory:
-            file_0_path = os.path.join(output_directory, "file_0.txt")
-            file_1_path = os.path.join(output_directory, "file_1.txt")
+        with tempfile.TemporaryDirectory() as temporary_directory:
+            file_0_path = os.path.join(temporary_directory, "file_0.txt")
+            file_1_path = os.path.join(temporary_directory, "file_1.txt")
 
             with open(file_0_path, "w") as f:
                 f.write("[1, 2, 3]")
@@ -309,15 +321,15 @@ class DatasetTestCase(BaseTestCase):
                 },
             )
 
-            dataset.to_cloud(project_name=project_name, bucket_name=bucket_name, output_directory=output_directory)
+            dataset.to_cloud(project_name=project_name, bucket_name=TEST_BUCKET_NAME, output_directory="a_directory")
 
         persisted_dataset = Dataset.from_cloud(
             project_name=project_name,
-            bucket_name=bucket_name,
-            path_to_dataset_directory=storage.path.join(output_directory, dataset.name),
+            bucket_name=TEST_BUCKET_NAME,
+            path_to_dataset_directory=storage.path.join("a_directory", dataset.name),
         )
 
-        self.assertEqual(persisted_dataset.path, f"gs://{bucket_name}{output_directory}/{dataset.name}")
+        self.assertEqual(persisted_dataset.path, f"gs://{TEST_BUCKET_NAME}/a_directory/{dataset.name}")
         self.assertEqual(persisted_dataset.id, dataset.id)
         self.assertEqual(persisted_dataset.name, dataset.name)
         self.assertEqual(persisted_dataset.hash_value, dataset.hash_value)
@@ -325,14 +337,13 @@ class DatasetTestCase(BaseTestCase):
         self.assertEqual({file.name for file in persisted_dataset.files}, {file.name for file in dataset.files})
 
         for file in persisted_dataset:
-            self.assertEqual(file.path, f"gs://{bucket_name}{output_directory}/{dataset.name}/{file.name}")
+            self.assertEqual(file.path, f"gs://{TEST_BUCKET_NAME}/a_directory/{dataset.name}/{file.name}")
 
     def test_to_cloud(self):
         """Test that a dataset can be uploaded to the cloud, including all its files and a serialised JSON file of the
         Datafile instance.
         """
         project_name = "test-project"
-        bucket_name = os.environ["TEST_BUCKET_NAME"]
         output_directory = "my_datasets"
 
         with tempfile.TemporaryDirectory() as temporary_directory:
@@ -352,24 +363,26 @@ class DatasetTestCase(BaseTestCase):
                 }
             )
 
-            dataset.to_cloud(project_name, bucket_name, output_directory)
+            dataset.to_cloud(project_name, TEST_BUCKET_NAME, output_directory)
 
             storage_client = GoogleCloudStorageClient(project_name)
 
             persisted_file_0 = storage_client.download_as_string(
-                bucket_name=bucket_name, path_in_bucket=storage.path.join(output_directory, dataset.name, "file_0.txt")
+                bucket_name=TEST_BUCKET_NAME,
+                path_in_bucket=storage.path.join(output_directory, dataset.name, "file_0.txt"),
             )
 
             self.assertEqual(persisted_file_0, "[1, 2, 3]")
 
             persisted_file_1 = storage_client.download_as_string(
-                bucket_name=bucket_name, path_in_bucket=storage.path.join(output_directory, dataset.name, "file_1.txt")
+                bucket_name=TEST_BUCKET_NAME,
+                path_in_bucket=storage.path.join(output_directory, dataset.name, "file_1.txt"),
             )
             self.assertEqual(persisted_file_1, "[4, 5, 6]")
 
             persisted_dataset = json.loads(
                 storage_client.download_as_string(
-                    bucket_name=bucket_name,
+                    bucket_name=TEST_BUCKET_NAME,
                     path_in_bucket=storage.path.join(output_directory, dataset.name, "dataset.json"),
                 )
             )
